# A Crash Course in R
# This code has been kindly contributed by Robin Edwards (from UCL CASA).
# There are many useful introductory guides out there to R, but below is the kind of thing I now wish Id been given when I first started using it  something with simple logically-progressive examples and minimal explanatory text. Copy the text below into a new script in R and run line-by-line to give a quick intro to many of Rs most basic principles and functionality. You can also download a text file with it here. It is by no means comprehensive, even at the most basic level, but still I hope someone finds it useful. You may want to look at RStudio as it is more user-friendly.

## A CRASH COURSE IN [R] PROGRAMMING
## Robin Edwards (geotheory.co.uk), March 2013
## Modified by Alexey Pindyurin, November 2014

# basic R environmental functions
x=3.14159; y='hello world'; z=TRUE # create some objects.
ls() # list the objects in the Workspace
print(y) # print information to R Console
rm(y) # remove an object
rm(list=ls()) # remove all
getwd() # find current working directory
setwd("/Users/robinedwards/Documents") # set working directory as preferred
print ( "R ignores the 'white-space' in command syntax" )

# use '?' for help on any R function (if its library is loaded in the session)
?max
??csv # search for a text string in R documentation library
library(help=utils) # get help on a particular package (list its functions)

# 'str is a powerful tool for investigating the underlying structure of any R object
str(max)

# CREATING AND MANIPULATING R OBJECTS

# assigning values to variables
n = 5 # is possible but
5 -> n
n <- 5
rm(n)

# R objects can be of various data types, but probably most common are numeric and character
num <- c(2.344, 45.565)

char <- 'any text string'

# create a VECTOR (array) using the 'c()' concatenate function
vec <- c(1, 23, 45, 3, 5, 90, 456, 23, 65, 12, 34, 2, 5)

# R vectors can be accessed in various ways using [ ] brackets
vec[3]
vec[3:6]
vec[ c(1,3,8) ]
vec[vec > 15]

# check a vector contains a value
5 %in% vec
12 %in% vec
345 %in% vec


# create a VECTOR (array) using the 'seq' function
vec <- seq(from=0, to=100, by=2);


# finding FIRST index position of a matching value/sting
x = c('one', 'five', 'two', 3, 'two')
match('two', x)
match(c('two','five'), x)

# a MATRIX is a 2D vector (essentially a vector of vectors) of MATCHING data type
matrx = matrix(1:15, 3, 5)
matrx
dim(matrx)
print(matrx)
t(matrx) # a matrix can be easily transposed

matrx2 = matrix(1:15, 3, 5, byrow=T)
matrx2

# a DATA.FRAME is like a matrix, but accomodates fields (columns) with different data types
DF <- data.frame(start=seq(1,135,by=5), end=seq(1,135,by=5)+4, LAD=rep(c(0,1,0,1,2,0), times=7:2))

# They can be viewed easily
View(DF)

# examine their internal stucture
str(DF)

# data interrogation with square brackets
DF[1,]
DF[2:3,]
DF[,1]
DF[2,1]

# plot
plot(DF[,1], DF[,3])

# data.frame and matrix objects can have field (column) and record (row) names
colnames(DF)
names(DF) # not for matrix objects
rownames(DF)

# interrogate data.frames by field name using the $ operator. the result is a simple vector
DF$start
DF$start[2]

# names can be reassigned
names(DF)
row.names(DF)
print(DF)

names(DF) <- c("start.Coord", "end.Coord", "Score")
DF

# check dimensions of vector/matrix/array/data.frame objects
length(vec)
dim(DF)
dim(arry)
nrow(DF)
ncol(DF)

# R has various inbuilt data.frame datasets used to illustrate how functions operate e.g.
data()
InsectSprays # this guide makes use of these datasets
warpbreaks

# examine contents
head(InsectSprays) # list the top records of a vector / matrix / d.f.
tail(InsectSprays, n=3) # bottom the 3
summary(InsectSprays) # summarise a data vector

# aggregate() is a powerful function for summarising categorical data
aggregate(InsectSprays$count, by=list(InsectSprays$spray), FUN=mean)
sumInsects
names(sumInsects)
print(sumInsects)

# subset/apply filter to a data.frame
warpbreaks[warpbreaks$wool=='A',] # by 1 condition
warpbreaks[warpbreaks$tension %in% c('L','M'),] # multiple conditions


# reorder a vector with sort
vec
sort(vec)

# or a dataframe with order
DF[order(DF$Score),]

# LOGICAL objects (booleans) are binary true/false objects that facilitate conditional data processing
bool <- c(TRUE, TRUE, FALSE, TRUE, TRUE)
bool

bool *1
sum(bool)

T + T
T * T
F * T

# query an object's data/structure type with 'class()'
class(bool)
class(num) # numeric is the default data type for number objects
class(as.integer(num)) # integer class exists but is not default
class(char) # character class
class("237") # numbers arent always numeric type
as.numeric("237") # but can be converted
as.character(237) # and vice verse

# Child-objects are often of different class to parents
class(DF)
class(DF[,2])
class(DF[,1])

# FACTOR objects are vectors of items that have been categorised by unique values
DF[,3] # 'Score' values

factr <- factor(DF[,3])

str(factr)
levels(factr)
table(factr)


# LOGICAL OPERATIONS
2 + 2 == 4 # '==' denotes value equality
3 <= 2 # less than or equal to
3 >= 2 # greater than or equal to
'string' == "string"
3 != 3 # NOT operator
c(4,2,6) == c(4,2,8) # vector comparisons return locical vectors
TRUE == T # 'T' and 'F' default as boolean shortcuts (until overwritten)
TRUE & TRUE # AND operator
TRUE | FALSE # OR operator
F | F

# IF/ELSE statement (used in most logical procedures)
x

if(x < 5){
print('x is less than 5')
} else {
print('x is not less than 5')
}


# LOOPING FUNCTIONS  very useful for handling repetitive operations

# 'FOR' loop
for(i in 1:10){
print(paste('number ',i))
}

# WHILE loop (be careful to include safeguards to prevent infinite loops)
i = 30
while(i > 0){
print(paste('number ',i))
i <- i - 3
}


# handling 'NA' values
x = 1:5
x
x[8] = 8
x
x[3] = NA
x
print(x) # sometimes functions will fail because of NA values
na.omit(x) # iterates full list but ignores NAs
x[na.omit(x)]
is.na(x) # alternatively
x[!is.na(x)]

# useful basic math functions
seq(-2, 2, by=.2) # sequence of equal difference
seq(length=10, from=-5, by=.2) # with range defined by vector length
rnorm(20, mean = 0, sd = 1) # random normal distribution
runif(20, min=0, max=100) # array of random numbers
sample(0:100, 20, replace=TRUE) # array of random integers
table(warpbreaks[,2:3]) # array summary stats (powerful summary tool)
min(vec)
max(vec)
range(vec)
mean(vec)
median(vec)

salary <- c(5,5,5,6,6,6,7,7,7,15,16,21,1500)
salary
mean(salary)

length(salary)
median(salary)

sum(vec)
prod(vec)
abs(-5) # magnitude
sd(rnorm(10)) # standard deviation
4^2 # square
sqrt(16) # square root
5%%3 # modulo (remainder after subtraction of any multiple)
6%%2
for(i in 1:100) if(i%%20==0) print(i) # useful for running an operation every nth cycle

# Importing and exporting data using comma-separated file
write.csv(DF, 'example.csv', rownames=F) # save to csv file
rm(DF)


DF <- read.delim('example.csv', sep=",", rownames=F)

# PLOTTING IN R

# some basic functionality
plot(1:10)
plot(sort(rnorm(100)), pch=16, cex=0.5) # specifying point and size respectively
plot(x=1:25, y=25:1, pch=1:25) # x & y inputs, and showing the available point symbols
plot(sin, -pi, 2*pi)
hist(rnorm(1000), breaks=50)
barplot(sumInsects$sum, names.arg = sumInsects$group)
pie(sumInsects$sum, labels = sumInsects$group)

# plots with more visual components are built up incrementally
y <- c(12,4,7,23,78,34)
plot(y, pch=17)
lines(y, col='#00FF00')
points(y+5, pch=16, col='red')
lines(sample(y), col='black') 


# END OF SCRIPT